/* Dynamic Array Library
 *
 * Author: Alptekin Kp
 * Implementation Date: 07.2003
 * Contact address: alpkup@programmer.net
 * 
 * These files and codes are fully distributable if no change is made on them.
 */

/* Implemented Functions:
 * 1. DynamicArray* newDynamicArray(void)
 * 2. boolean addElement(DynamicArray* head , void* element)
 * 3. void* getElementAt(DynamicArray* head , int elementNo)
 * 4. boolean removeElement(DynamicArray* head)
 * 5. int getElementCount(DynamicArray* head)
 * 6. int getArrayListLength(DynamicArray* head)
 *
 * Additional functions (See "Important Notice 2" below):
 * 1. boolean addElementAt(DynamicArray* head , void* element , int position , boolean overwrite)
 * 2. boolean removeElementAt(DynamicArray* head , int position)
 */

/* Important Notice:
 * Change MAX_HASH to best satisfy your needs (Default is 100)
 * If getArrayListLength function returns a high value 
 * try to increase MAX_HASH value to increase performance
 */
/* Important Notice 2:
 * Do not use additional functions if not absolutely necessary
 * Or be sure to use properly (with proper parameters)
 * Otherwise some anormalities may occur
 * (See function prototypes below for more explanation)
 */


// Includes
#include <stdlib.h>


#ifndef DYNAMICARRAY_H
#define DYNAMICARRAY_H

// Maximum array size
// Change this according to the order of your need
#define MAX_HASH 100


// Defines
#ifdef NULL
#undef NULL
#endif
#define NULL 0


// Simple boolean type for c
//#ifndef boolean
//#define boolean
typedef short boolean;
#define TRUE 1
#define FALSE 0
//#endif


// Array structure
struct arrs
{
	void** array;
	struct arrs* next;
	int currentSize;
};
typedef struct arrs DynamicArray;


/********************************************************/
// Function prototypes

// Creates a new dynamic array
// Memory is automatically allocated
DynamicArray* newDynamicArray(void);

// Adds given element to the first empty position
// in the array list starting from head
// Creates a new array if current arrays are all full
// Returns FALSE if given head array is NULL
boolean addElement(DynamicArray* head , void* element);

// Returns the element at the given index
// Starting from given head array
// Searches all arrays in the list
// Returns NULL if given head array is NULL
void* getElementAt(DynamicArray* head , int elementNo);

// Removes the last element in the array
// Returns FALSE if given head array is NULL
boolean removeElement(DynamicArray* head);

// Returns the total sizes of all arrays starting from head
// Returns -1 if given head array is NULL
int getElementCount(DynamicArray* head);

// Returns the length of the linked list of dynamic arrays
// Returns -1 if given head array is NULL
// If this function returns a high value, try to increase MAX_HASH value
// defined in "DynamicArray.h" to increase performance
int getArrayListLength(DynamicArray* head);


/*******************************************************************/
// Additional functions

/* Important Notice:
 * Do not use following functions if not absolutely necessary
 * Or be sure to use properly (with proper parameters)
 * Otherwise some anormalities may occur
 */

// Adds an element to the given position
// Creates new arrays if necessary
// Returns FALSE if given head array is NULL
// Returns FALSE if overwrite is FALSE and there exists an item at given position
// Does not count for current size (element count)
boolean addElementAt(DynamicArray* head , void* element , int position , boolean overwrite);

// Removes the element at the given position
// Returns FALSE if given head array is NULL
// Be sure not to give a bad position in order not to get 
// "ACCESS VIOLATION" error
boolean removeElementAt(DynamicArray* head , int position);

/********************************************************/

#endif DYNAMICARRAY_H
